/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which
 * are not licensed under the terms of the GPL, given that they satisfy one
 * or more of the following conditions:
 * 1) Explicit license is granted to the ProM and ProMimport programs for
 *    usage, linking, and derivative work.
 * 2) Carte blance license is granted to all programs developed at
 *    Eindhoven Technical University, The Netherlands, or under the
 *    umbrella of STW Technology Foundation, The Netherlands.
 * For further exemptions not covered by the above conditions, please
 * contact the author of this code.
 * 
 */
package org.processmining.plugins.fuzzymodel.miner.replay;

import org.deckfour.xes.model.XEvent;
import org.processmining.models.graphbased.directed.fuzzymodel.FMNode;



/**
 * @author christian
 * @author Jiafei Li (jiafei@jlu.edu.cn)
 *
 */
public class DetailNodeAnalysis implements Comparable<DetailNodeAnalysis> {
	
	public enum SimplificationType {
		PRESERVED, CLUSTERED, REMOVED;
	}
	
	protected FMNode node;
	protected XEvent event;
	protected SimplificationType type;
	
	public DetailNodeAnalysis(FMNode node, XEvent event, SimplificationType type) {
		this.node = node;
		this.event = event;
		this.type = type;
	}
	
	public FMNode getNode() {
		return node;
	}
	
	public XEvent getEvent() {
		return event;
	}
	
	public SimplificationType getType() {
		return type;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(DetailNodeAnalysis dna) {
		double sigDif = node.getSignificance() - dna.node.getSignificance();
		if(sigDif == 0) {
			return 0;
		} else if(sigDif > 0) {
			return 1;
		} else {
			return -1;
		}
	}

}
